unit FileProc;

interface

uses
  Windows, SysUtils, LockApp;

type
  ASCIIString = type AnsiString(20127 {437});
  CyrillicString = type AnsiString(1251);
  WesternString = type AnsiString(1252);
  EasternString = type AnsiString(1250);
  HebrewString = type AnsiString(1255);
  ArabicString = type AnsiString(1256);
  GreekString = type AnsiString(1253);
  JapanString = type AnsiString(932);
  KoreanString = type AnsiString(949);
  VietnameseString = type AnsiString(1258);
  ChineseSimplifiedString = type AnsiString(936);
  ChineseTraditionalString = type AnsiString(950);

  TUnicodeTest =
    (sInvalid, sAnsi, sUnicode, sOther, sASCII, sCyrillic, sWestern, sEastern,
    sHebrew, sArabic, sGreek, sJapan, sKorean, sVietnamese, sChineseSimplified,
    sChineseTraditional);
  TAccessTest =
    (accDir, accNotFound, accEnabled, accDenied);

const
  SUnicodeTest: array[TUnicodeTest] of string =
    ('Invalid', 'ANSI', 'Unicode', 'Other', 'ascii', 'Cyrillic', 'Western',
    'Central/Eastern', 'Hebrew', 'Arabic', 'Greek', 'Shift JIS', 'Korean',
    'Vietnamese', 'Chinese Simplified', 'Chinese Traditional');
  SAccessTest: array[TAccessTest] of string =
    ('', 'Not found', 'Access', 'Locked');

function FUnicodeTest(const fn: string; CheckExists: Boolean = True): TUnicodeTest; inline;
function FUnicodeTestEx(const fn: string): TUnicodeTest; inline;
function FReadTest(const fn: string): TAccessTest; inline;
function FWriteTest(const fn: string): TAccessTest; inline;
function FLockApp(const fn: string): string;

implementation

//-----------------------------------------------------
procedure FExistTest(const fn: string; var fExist, fDir, fRO: boolean); inline;
var
  h: THandle;
  fd: TWin32FindData;
begin
  fExist:= false;
  fDir:= false;
  fRO:= false;
  if fn='' then Exit;
  h:= FindFirstFile(PChar(fn), fd);
  if h=INVALID_HANDLE_VALUE then Exit;
  fExist:= true;
  fDir:= (fd.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY)<>0;
  fRO:= (fd.dwFileAttributes and FILE_ATTRIBUTE_READONLY)<>0;
  Windows.FindClose(h);
end;

//-----------------------------------------------------
function FReadTest(const fn: string): TAccessTest;
var
  h: THandle;
  fExist, fDir, fRO: boolean;
begin
  FExistTest(fn, fExist, fDir, fRO);
  if not fExist then begin Result:= accNotFound; Exit end;
  if fDir then begin Result:= accDir; Exit end;

  h:= CreateFile(PChar(fn),
    GENERIC_READ,
    0{FILE_SHARE_READ},
    nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if h<>INVALID_HANDLE_VALUE then
    Result:= accEnabled
  else
    Result:= accDenied;
  if h<>INVALID_HANDLE_VALUE then
    CloseHandle(h);
end;

//-----------------------------------------------------
function FWriteTest(const fn: string): TAccessTest;
var
  h: THandle;
  fExist, fDir, fRO: boolean;
begin
  FExistTest(fn, fExist, fDir, fRO);
  if not fExist then begin Result:= accNotFound; Exit end;
  if fDir then begin Result:= accDir; Exit end;

  if fRO then
    FileSetReadOnly(fn, false);

  h:= CreateFile(PChar(fn),
    GENERIC_READ or GENERIC_WRITE,
    0{FILE_SHARE_READ},
    nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if h<>INVALID_HANDLE_VALUE then
    Result:= accEnabled
  else
    Result:= accDenied;
  if h<>INVALID_HANDLE_VALUE then
    CloseHandle(h);

  if fRO then
    FileSetReadOnly(fn, true);
end;

//-----------------------------------------------------
function IsNameLong(const fn: string): boolean;
var
  h: THandle;
  fd: TWin32FindData;
  fnA, dir: string;
begin
  Result:= false;
  dir:= ExtractFileDir(fn);
  fnA:= ExtractFileName(fn);
  h:= FindFirstFile(PChar(dir+'\*.*'), fd);
  if h<>INVALID_HANDLE_VALUE then
  repeat
    if string(fd.cFileName)='.' then Continue;
    if StrIComp(fd.cFileName, PChar(fnA))=0 then
      begin Result:= true; Break end;
  until not FindNextFile(h, fd);
  Windows.FindClose(h);
end;

//-----------------------------------------------------
function IsNameShort(const fn: string): boolean;
var
  h: THandle;
  fd: TWin32FindData;
  fnA, dir: string;
begin
  Result:= false;
  dir:= ExtractFileDir(fn);
  fnA:= ExtractFileName(fn);
  h:= FindFirstFile(PChar(dir+'\*.*'), fd);
  if h<>INVALID_HANDLE_VALUE then
  repeat
    if string(fd.cFileName)='.' then Continue;
    if (StrIComp(fd.cFileName, fd.cAlternateFileName)<>0) and
       (StrIComp(fd.cAlternateFileName, PChar(fnA))=0) then
      begin Result:= true; Break end;
  until not FindNextFile(h, fd);
  Windows.FindClose(h);
end;

//-----------------------------------------------------
function IsNameUnicode(const fn: string): boolean;
var
  h: THandle;
  fdW: TWin32FindData;
  fnS: string;
  dirW: string;
begin
  Result:= false;
  fnS:= ExtractFileName(fn);
  dirW:= ExtractFileDir(fn);
  h:= FindFirstFile(PWChar(dirW+'\*.*'), fdW);
  if h<>INVALID_HANDLE_VALUE then
  repeat
    if WideString(fdW.cFileName)=WideChar('.') then Continue;
    if UpperCase(fnS) = UpperCase(fdW.cAlternateFileName) then
      begin Result:= true; Break end;
  until not FindNextFile(h, fdW);
  Windows.FindClose(h);
end;

//-----------------------------------------------------
function FUnicodeTest(const fn: string; CheckExists: Boolean = True): TUnicodeTest;
var
  fEx, fDir, fRO: boolean;
begin
  if CheckExists then
    FExistTest(fn, fEx, fDir, fRO)
  else
    fEx := True;
  if fEx then
  begin
    if fn = string(AnsiString(fn)) then Result:= sAnsi else
     Result:= sUnicode;
  end
  else
    Result:= sInvalid;
end;

//-----------------------------------------------------
function FUnicodeTestEx(const fn: string): TUnicodeTest;
var
  I: Integer;
begin
    if fn = string(ASCIIString(fn)) then Result:= sASCII else
      if fn = string(WesternString(fn)) then Result:= sWestern else
        if fn = string(CyrillicString(fn)) then Result:= sCyrillic else
          if fn = string(EasternString(fn)) then Result:= sEastern else
            if fn = string(GreekString(fn)) then Result:= sGreek else
              if fn = string(HebrewString(fn)) then Result:= sHebrew else
                if fn = string(ArabicString(fn)) then Result:= sArabic else
                  //if fn = string(ChineseTraditionalString(fn)) then Result:= sChineseTraditional else
                    //if fn = string(ChineseSimplifiedString(fn)) then Result:= sChineseSimplified else
                      //if fn = string(JapanString(fn)) then Result:= sJapan else
                        //if fn = string(KoreanString(fn)) then Result:= sKorean else
                          //if fn = string(VietnameseString(fn)) then Result:= sVietnamese else
                            Result:= sOther;
end;

function FLockApp(const fn: string): string;
begin
  Result := '';
  try
    if (FWriteTest(fn) = accDenied) or (FReadTest(fn) = accDenied) then
      Result := LockApp.GetLockApp(fn);
  except
    // do nothing
  end;
end;

end.
